import Default from './Default';
import DefaultList from './DefaultList';
import DefaultDetail from './DefaultDetail';
import DefaultEdit from './DefaultEdit';
import Index from './IndexView';
import Login from './Login';
import Logout from './Logout';
import OutboxList from './OutboxList';
import Loading from './Loading';
import NotFound from './NotFound';
import Server from './Server';

export {
    Default,
    DefaultList,
    DefaultDetail,
    DefaultEdit,
    Index,
    Login,
    Logout,
    OutboxList,
    Loading,
    NotFound,
    Server
};
